"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dotup_typescript_yeoman_generators_1 = require("dotup-typescript-yeoman-generators");
const lodash_1 = __importDefault(require("lodash"));
const TypescriptGenerator_1 = require("../ts/TypescriptGenerator");
const validate_npm_package_name_typed_1 = __importDefault(require("validate-npm-package-name-typed"));
const MyQuestions_1 = require("./MyQuestions");
const TypescriptQuestions_1 = require("../ts/TypescriptQuestions");
const inquirer = require("inquirer");
class MyGenerator extends dotup_typescript_yeoman_generators_1.BaseGenerator {
    constructor(args, options) {
        super(args, lodash_1.default.merge(options, { 'sharedOptions': new dotup_typescript_yeoman_generators_1.SharedOptions() }));
        super.registerMethod(this);
        this.writeOptionsToAnswers(MyQuestions_1.MyQuestions);
        this.destinationRoot(this.destinationPath());
    }
    async initializing() {
        // Project name
        this.addQuestion(new dotup_typescript_yeoman_generators_1.Question(MyQuestions_1.MyQuestions.projectName, {
            message: 'Project Name',
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.input,
            default: this.getDefaultProjectName(),
            validate: (v) => this.validateString(v),
            acceptAnswer: v => {
                const accept = validate_npm_package_name_typed_1.default(v.toString()).validForNewPackages;
                if (!accept) {
                    this.logRed(`${v} is not a valid package name.`);
                }
                return true;
            },
            When: _ => this.tryGetAnswer(MyQuestions_1.MyQuestions.projectName) === undefined
        }));
        // Invalid project name
        this.addQuestion(new dotup_typescript_yeoman_generators_1.Question(MyQuestions_1.MyQuestions.invalidProjectName, {
            message: 'Continue anyway?',
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.confirm,
            default: 'N',
            acceptAnswer: accepted => {
                if (!accepted) {
                    // Ask again for the project name
                    this.currentStep = MyQuestions_1.MyQuestions.projectName;
                }
                return accepted;
            },
            When: () => {
                const name = this.tryGetAnswer(MyQuestions_1.MyQuestions.projectName);
                return !validate_npm_package_name_typed_1.default(name).validForNewPackages;
            }
        }));
        // Create folder?
        this.addQuestion(new dotup_typescript_yeoman_generators_1.Question(MyQuestions_1.MyQuestions.createFolder, {
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.confirm,
            message: () => `Create folder '${this.answers.projectName}' ?`,
            default: 'Y',
            When: () => !this.destinationIsProjectFolder(this.answers.projectName),
            acceptAnswer: accepted => {
                if (accepted) {
                    // Create new root
                    this.destinationRoot(this.destinationPath(this.answers.projectName));
                    this.logGreen(`Destination path changed to ${this.destinationPath()}`);
                }
                return accepted;
            }
        }));
        this.addQuestion(new dotup_typescript_yeoman_generators_1.ConfirmQuestion(MyQuestions_1.MyQuestions.useGit, 'Configure git?'));
        // Use github?
        this.addQuestion(new dotup_typescript_yeoman_generators_1.StoreQuestion(MyQuestions_1.MyQuestions.useGithub, {
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.confirm,
            message: 'Configure github?',
            When: _ => this.answers.useGit.toString() === 'true'
        }));
    }
    async prompting() {
        if (this.skipGenerator) {
            return;
        }
        await super.prompting();
        this.compose('generator-dotup-npm-package/generators/app');
        // Typescript generator
        this.composeWith({
            Generator: TypescriptGenerator_1.TypescriptGenerator,
            path: require.resolve('../ts/index')
        }, {
            [TypescriptQuestions_1.TypescriptQuestions.sourcePath]: 'src',
            [TypescriptQuestions_1.TypescriptQuestions.targetPath]: 'dist',
            [TypescriptQuestions_1.TypescriptQuestions.testPath]: 'test',
            [TypescriptQuestions_1.TypescriptQuestions.docsPath]: 'docs'
        });
        if (this.answers.useGit.toString() === 'true') {
            // TODO: remove rootPath
            // (<any>this.answers).rootPath = this.destinationPath(),
            this.compose('generator-dotup-git/generators/app');
        }
        if (this.answers.useGithub && this.answers.useGithub.toString() === 'true') {
            // Load git generator
            this.compose('generator-dotup-github/generators/app');
        }
        this.compose('generator-dotup-gulp/generators/app', true, this.answers);
        // case ProjectType.ts_yo_generator:
        //   this.composeWith(
        //     <any>{
        //       Generator: YeomanGeneratorGenerator,
        //       path: require.resolve('../ts-yogen/index')
        //     },
        //     {
        //       [TsQuestions.projectName]: this.answers.projectName,
        //       [TsQuestions.sourcePath]: 'src',
        //       [TsQuestions.targetPath]: 'generators',
        //       [TsQuestions.testPath]: 'test',
        //       [TsQuestions.docsPath]: 'docs',
        //       [TsQuestions.mainFile]: 'app/index.js',
        //       [TsQuestions.typesFile]: 'app/index.d.ts'
        //     }
        //   );
        //   break;
    }
    async end() {
        this.logGreen('Your project is ready.');
        const q = {
            name: 'vscode',
            message: 'Should I start vscode?',
            default: 'Y',
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.confirm
        };
        const result = await inquirer.prompt(q);
        if (result.vscode === true) {
            this.spawnCommandSync('code', [this.destinationPath()]);
        }
    }
}
exports.MyGenerator = MyGenerator;

//# sourceMappingURL=MyGenerator.js.map
