"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const dotup_typescript_yeoman_generators_1 = require("dotup-typescript-yeoman-generators");
const validate_npm_package_name_typed_1 = __importDefault(require("validate-npm-package-name-typed"));
const TypescriptAppGenerator_1 = require("../ts-app/TypescriptAppGenerator");
const TypescriptLibGenerator_1 = require("../ts-lib/TypescriptLibGenerator");
const TypescriptQuestions_1 = require("./TypescriptQuestions");
class TypescriptGenerator extends dotup_typescript_yeoman_generators_1.BaseGenerator {
    constructor(args, options) {
        super(args, options);
        this.registerMethod(this);
        this.writeOptionsToAnswers(TypescriptQuestions_1.TypescriptQuestions);
    }
    async initializing() {
        // Project type
        this.addQuestion(new dotup_typescript_yeoman_generators_1.StoreQuestion(TypescriptQuestions_1.TypescriptQuestions.projectType, {
            message: `Project type?`,
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.list,
            choices: [
                {
                    name: 'Node application (Typescript)',
                    value: dotup_typescript_yeoman_generators_1.ProjectType.app
                },
                {
                    name: 'Node library (Typescript)',
                    value: dotup_typescript_yeoman_generators_1.ProjectType.lib
                }
            ],
            When: a => this.tryGetAnswer(TypescriptQuestions_1.TypescriptQuestions.projectType) === undefined
        }));
        // Project name
        this.addQuestion(new dotup_typescript_yeoman_generators_1.Question(TypescriptQuestions_1.TypescriptQuestions.projectName, {
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.input,
            message: 'Project Name',
            default: this.getDefaultProjectName(),
            validate: (v) => this.validateString(v),
            acceptAnswer: v => {
                const accept = validate_npm_package_name_typed_1.default(v.toString()).validForNewPackages;
                if (!accept) {
                    this.logRed(`${v} is not a valid package name.`);
                }
                return true;
            },
            When: () => {
                return this.tryGetAnswer(TypescriptQuestions_1.TypescriptQuestions.projectName) === undefined;
            }
        }));
        this.addQuestion(new dotup_typescript_yeoman_generators_1.OptionalQuestion(TypescriptQuestions_1.TypescriptQuestions.invalidProjectName, {
            type: dotup_typescript_yeoman_generators_1.InquirerQuestionType.confirm,
            message: 'Continue anyway?',
            default: 'N',
            acceptAnswer: accepted => {
                if (!accepted) {
                    // Ask again for the project name
                    this.currentStep = TypescriptQuestions_1.TypescriptQuestions.projectName;
                }
                return accepted;
            },
            When: () => {
                const name = this.tryGetAnswer(TypescriptQuestions_1.TypescriptQuestions.projectName);
                return !validate_npm_package_name_typed_1.default(name).validForNewPackages;
            }
        }));
    }
    async prompting() {
        const result = await super.prompting();
        // Application type generator
        switch (this.answers.projectType) {
            case dotup_typescript_yeoman_generators_1.ProjectType.app:
                this.composeWith({
                    Generator: TypescriptAppGenerator_1.TypescriptAppGenerator,
                    path: require.resolve('../ts-app/index')
                }, {
                    // [TsQuestions.projectName]: this.sharedOptions.getAnswer(TsQuestions.projectName),
                    // 'sharedOptions': this.sharedOptions,
                    [TypescriptQuestions_1.TypescriptQuestions.sourcePath]: this.answers.sourcePath,
                    [TypescriptQuestions_1.TypescriptQuestions.targetPath]: this.answers.targetPath,
                    [TypescriptQuestions_1.TypescriptQuestions.testPath]: this.answers.testPath,
                    [TypescriptQuestions_1.TypescriptQuestions.docsPath]: this.answers.docsPath,
                    [TypescriptQuestions_1.TypescriptQuestions.mainFile]: this.answers.mainFile || 'app.js',
                    [TypescriptQuestions_1.TypescriptQuestions.typesFile]: this.answers.typesFile || 'app.d.ts'
                });
                break;
            case dotup_typescript_yeoman_generators_1.ProjectType.lib:
                this.composeWith({
                    Generator: TypescriptLibGenerator_1.TypescriptLibGenerator,
                    path: require.resolve('../ts-lib/index')
                }, {
                    // [TsQuestions.projectName]: this.answers.projectName,
                    // 'sharedOptions': this.sharedOptions,
                    [TypescriptQuestions_1.TypescriptQuestions.sourcePath]: this.answers.sourcePath,
                    [TypescriptQuestions_1.TypescriptQuestions.targetPath]: this.answers.targetPath,
                    [TypescriptQuestions_1.TypescriptQuestions.testPath]: this.answers.testPath,
                    [TypescriptQuestions_1.TypescriptQuestions.docsPath]: this.answers.docsPath,
                    [TypescriptQuestions_1.TypescriptQuestions.mainFile]: this.answers.mainFile || 'index.js',
                    [TypescriptQuestions_1.TypescriptQuestions.typesFile]: this.answers.typesFile || 'index.d.ts'
                });
                break;
            // Only basic files
            case dotup_typescript_yeoman_generators_1.ProjectType.blank:
                break;
            // case ProjectType.ts_yo_generator:
            //   this.composeWith(
            //     <any>{
            //       Generator: YeomanGeneratorGenerator,
            //       path: require.resolve('../ts-yogen/index')
            //     },
            //     {
            //       [TsQuestions.projectName]: this.answers.projectName,
            //       [TsQuestions.sourcePath]: 'src',
            //       [TsQuestions.targetPath]: 'generators',
            //       [TsQuestions.testPath]: 'test',
            //       [TsQuestions.docsPath]: 'docs',
            //       [TsQuestions.mainFile]: 'app/index.js',
            //       [TsQuestions.typesFile]: 'app/index.d.ts'
            //     }
            //   );
            //   break;
            default:
                throw new Error('Project type not implemented');
        }
    }
    async writing() {
        await super.writing();
    }
    // :( Refactor !
    // loadTemplateFiles(): void {
    //   super.loadTemplateFiles();
    //   const analyser = new ProjectPathAnalyser((...args) => this.templatePath(...args));
    //   const files = analyser.getDeepFiles(this.templatePath('..', '..', 'shared'));
    //   files.forEach(f => f.targetPath = this.destinationPath('tools', 'gulp', path.basename(f.targetPath)));
    //   this.projectFiles.templateFiles.push(...files);
    // }
    async end() {
        // this.logGreen('Your project is ready.');
        // const q = {
        //   name: 'vscode',
        //   message: 'Should I start vscode?',
        //   default: 'Y',
        //   type: InquirerQuestionType.confirm
        // };
        // const result: ITypedProperty<boolean> = await inquirer.prompt(q);
        // if (result.vscode === true) {
        //   this.spawnCommandSync('code', [this.destinationPath()]);
        // }
    }
}
exports.TypescriptGenerator = TypescriptGenerator;

//# sourceMappingURL=TypescriptGenerator.js.map
